import chimera, numpy
from chimera import runCommand
from numpy import zeros, array

from StructBio.Scenographics.solids import Solids
from StructBio.Utilities.miscFunctions import residueKDhColor

#==============================================================================
#  ResidueCenter  Class
#==============================================================================
class ResidueCenter(object):
    def __init__(self, resObj):
        self.residue = resObj

    def coord(self):
        # We return coordinates of the alpha carbon for GLY and for any residue
        # with a missing side chain (for example, some end-of-chain residues in 1ZNJ)
        cbExists = self.residue.findAtom('CB')
        if not cbExists: return array(self.residue.findAtom('CA').coord())
        
        retVal = zeros(3)
        aCount = 0
        for a in self.residue.atoms:
            if a.name in ("CA", "C", "N", "O"): continue
            retVal += array(a.coord())
            aCount += 1
        return retVal/aCount



#==============================================================================
# Mainline
#==============================================================================

runCommand("set bg_color white")

pdbID = raw_input("Type in four character PDB Identifierd: \n")

prot = chimera.openModels.open(pdbID, type = "PDB")[0]

kdSpheres = Solids("Spheres")

for r in prot.residues:
    if r.findAtom("CA"):
        rc = ResidueCenter(r)
        kdSpheres.addSphere(rc.coord(), 1.0, residueKDhColor(r))

kdSpheres.display()
