import chimera, numpy, Tkinter as tk
from chimera import runCommand
from chimera.baseDialog import ModelessDialog
from chimera.baseDialog import ModalDialog
from chimera import runCommand
from numpy import zeros, array
from Tkinter import Entry

from StructBio.Utilities.miscFunctions import residueKDhColor
from StructBio.Utilities.shellNeighbors import Shell
from StructBio.Graphs.graph import Graph, Node, Edge

#------------------------------------------------------------------------------
# InputPDBidDialog modal dialog class
class InputPDBidDialog(ModalDialog):
    title = "User Input"
    buttons = ('OK', 'Cancel')
    default = 'OK'
    oneshot = True
    help = False

    def __init__(self):
        ModalDialog.__init__(self)
        
    def fillInUI(self, parent):
        
        #-------------------------------------------------------------------------
        # Setting up a entry box: 
        tk.Label(parent, text =  " Type PDB ID:  ").grid(row = 0, column = 0,
                                                          sticky = tk.W)
       
        self.entryVar = tk.StringVar()   
        self.entryPDBid = Entry(parent, textvariable = self.entryVar, width = 8,
                                takefocus = 1, bg = "white")
        self.entryPDBid.grid(row = 1, pady = 10, padx = 4, column = 0, sticky = tk.E)
        

        #-------------------------------------------------------------------------
        # Setting up a scale:
        
        scaleText = tk.Label(parent, text =  " Input distance threshold:  ")
        scaleText.grid(row = 2, column = 0, pady = 2, sticky = tk.W)
        
        self.scaleVar = tk.DoubleVar()
        scale = tk.Scale(parent, variable = self.scaleVar, orient = tk.HORIZONTAL,
                         length = 100, from_ = 1.0, to = 12.0, resolution = 0.1,
                         troughcolor = "goldenrod1")
        scale.grid(row = 3, column = 0, padx = 2, sticky = tk.E)
        scale.set(8.0)

        self.entryPDBid.focus_set()

        
    def OK(self):
        self.resultPDBid = self.entryVar.get()
        self.resultThreshold = float(self.scaleVar.get())        
        ModalDialog.Cancel(self, value=None)

    def Cancel(self):
        self.resultPDBid = None 
        ModalDialog.Cancel(self, value=None)
        
#------------------------------------------------------------------------------
# ResNetDialog class (modeless dialog)
class ResNetDialog(ModelessDialog):
    buttons = ("Close")
    title = "Residue Networks"
    help = False

    def fillInUI(self, parent):
        tk.Label(parent, text =  "  Specify the links to be shown:  ", anchor = tk.W).grid(row = 0, column = 0)
        self.checkedPhobicPhobic = tk.IntVar()
        self.checkedPhobicPhilic = tk.IntVar()
        self.checkedPhilicPhilic = tk.IntVar()
        self.checkedRibbon = tk.IntVar()
        
        cbPhobicPhobic = tk.Checkbutton(parent, text = "Hydrophobic-Hydrophobic  ",
                                        variable = self.checkedPhobicPhobic,
                                        command = self.showPhobicPhobic, indicatoron = 1)
        cbPhobicPhobic.grid(row = 1, column = 0)
        cbPhobicPhobic.select()
        
        cbPhobicPhilic = tk.Checkbutton(parent, text = "Hydrophobic-Hydrophilic  ",
                                        variable = self.checkedPhobicPhilic,
                                        command = self.showPhobicPhilic, indicatoron = 1)
        cbPhobicPhilic.grid(row = 1, column = 1)
        cbPhobicPhilic.select()
        
        cbPhilicPhilic = tk.Checkbutton(parent, text = "Hydrophilic-Hydrophilic  ",
                                        variable = self.checkedPhilicPhilic,
                                        command = self.showPhilicPhilic, indicatoron = 1)
        cbPhilicPhilic.grid(row = 1, column = 2)
        cbPhilicPhilic.select()

        #tk.Label(parent, text =  "Show ribbon:  ").grid(row = 2, column = 0, sticky = tk.E)
        cbRibbon = tk.Checkbutton(parent, text = "Show Ribbon",
                                        variable = self.checkedRibbon,
                                        command = self.showRibbon, indicatoron = 1)
        cbRibbon.grid(row = 2, column = 1, sticky = tk.W)
        cbRibbon.select()


    def showPhobicPhobic(self):
        if self.checkedPhobicPhobic.get():
            phobic_phobicSG.solids.show()
        else:
            phobic_phobicSG.solids.hide()

    def showPhobicPhilic(self):
        if self.checkedPhobicPhilic.get():
            phobic_philicSG.solids.show()
        else:
            phobic_philicSG.solids.hide()

    def showPhilicPhilic(self):
        if self.checkedPhilicPhilic.get():
            philic_philicSG.solids.show()
        else:
            philic_philicSG.solids.hide()
            
    def showRibbon(self):
        if self.checkedRibbon.get():
            runCommand("modeldisplay #" + str(prot.id))
        else:
            runCommand("~modeldisplay #" + str(prot.id))            


#==============================================================================
#  ResidueCenter  Class
#==============================================================================
class ResidueCenter(object):
    def __init__(self, resObj):
        self.residue = resObj
        self.label = ":" + str(resObj.id.position) + "." + resObj.id.chainId
        self.coords = zeros(3)

        # Note: self.coords will be the coordinates of the alpha carbon for GLY
        # and for any residue with a missing side chain (for example,
        # some end-of-chain residues in 1ZNJ)
        cbExists = self.residue.findAtom('CB')
        if not cbExists:
            self.coords = array(self.residue.findAtom('CA').coord())
        else:
            aCount = 0
            for a in self.residue.atoms:
                if a.name in ("CA", "C", "N", "O"): continue
                self.coords += array(a.coord())
                aCount += 1
            self.coords = self.coords/aCount

    def coord(self):
        return self.coords

#==============================================================================
# Mainline
#==============================================================================
runCommand("set bg_color white")

userDialog = InputPDBidDialog()
userDialog.run(chimera.tkgui.app)
if userDialog.resultPDBid:
    prot = chimera.openModels.open(userDialog.resultPDBid, type = "PDB")[0]
    resDistThreshold = userDialog.resultThreshold

resNetwork = Graph()

resCntr_L = []
# Build a list of ResidueCenter objects and generate neighbor dictionary.
# We can also filter with respect to hydrophobicity.
for r in prot.residues:
    if not r.kdHydrophobicity: continue # Skip unusual residues not having a KD value.
    if r.findAtom("CA"):                # Checking for standard residue.
        resCntr_L.append(ResidueCenter(r))

rCshell = Shell(resCntr_L, 0.0, resDistThreshold)

# Put in spheres and spindles.
for rC in resCntr_L:
    resNetwork.addNode(rC.label, Node(rC.residue.kdHydrophobicity, rC.coords,
                                      0.7, residueKDhColor(rC.residue)))       
    for nbrC in rCshell.getAtomsInShell(rC):
        # We do not want to put an edge in twice:
        if nbrC.residue.id.chainId > rC.residue.id.chainId: continue
        if (nbrC.residue.id.chainId == rC.residue.id.chainId and
            nbrC.residue.id.position > rC.residue.id.position): continue
        spindColor = tuple((array(residueKDhColor(rC.residue)) +
                            array(residueKDhColor(nbrC.residue)))/2.0)
        resNetwork.addEdge(rC.label, nbrC.label, Edge(0.0, 0.3, spindColor))


phobic_phobicSG = resNetwork.getSubgraph_NodeWeightsWithinRange(0.0, 5.0, "Phobic_Phobic subgraph")
phobic_phobicSG.solids.display()

philic_philicSG = resNetwork.getSubgraph_NodeWeightsWithinRange(-5.0, 0.0, "Philic_Philic subgraph")
philic_philicSG.solids.display()
            
homogenousEdges_L = phobic_phobicSG.getArcTupleList()
homogenousEdges_L.extend(philic_philicSG.getArcTupleList())
phobic_philicSG = resNetwork.getSubgraph_EdgesRemoved(homogenousEdges_L, "Phobic_Philic subgraph")
phobic_philicSG.solids.display()

rnDialog = ResNetDialog()

