import chimera
from chimera import runCommand

from StructBio.Utilities.miscFunctions import userSpecificationOfChains
from StructBio.StructureOverlap.sequencesLocalAlignment import SequencesLocalAlignment

from crn import RRcontacts, ContactRearrangementNetwork
#------------------------------------------------------------------------------
def getStandardResidues(prot):
    r_L = []
    for r in prot.residues:
        if r.type in ["ALA", "ARG", "ASN", "ASP", "CYS",
                      "GLU", "GLN", "GLY", "HIS", "ILE",
                      "LEU", "LYS", "MET", "PHE", "PRO",
                      "SER", "THR", "TRP", "TYR", "VAL"] and r.findAtom('CA'): r_L.append(r)
    return r_L

#==============================================================================
# Mainline
#==============================================================================
runCommand("set bg_color white")

# Atom counts (side chain only) used to determine radii of nodes in graph:
atomCounts_D = {"ALA":1, "ARG":7, "ASN":4, "ASP":4, "CYS":2,
                "GLU":5, "GLN":5, "GLY":0, "HIS":6, "ILE":4,
                "LEU":4, "LYS":5, "MET":4, "PHE":7, "PRO":3,
                "SER":2, "THR":3, "TRP":10, "TYR":8, "VAL":3}

(protI, protA) = userSpecificationOfChains(2)

# Ideally both chains should have the same primary sequence but this might not guaranteed!
# It is possible that one chain has missing residues that do occur in the other chain.
# It is possible that one chain has one or more residue mutations! (Consider 1E0S and 1HFV).
# So, it is necessary to do a sequence alignment working with standard residues each
# containing an alpha carbon. This last requirement seems strange but is needed to get
# past a PDB file with an erroneous residue entry...

rI_L = getStandardResidues(protI)
rA_L = getStandardResidues(protA)
alignedPairs = SequencesLocalAlignment(rI_L, rA_L)
[rI_L, rA_L] = zip(*alignedPairs.alignment_L)

# Get rid of residues that do not match:  
residuesI_L = []
residuesA_L = []
for ix in range(len(rI_L)):
    if rI_L[ix].type == rA_L[ix].type:
        residuesI_L.append(rI_L[ix])
        residuesA_L.append(rA_L[ix])
    
crn = ContactRearrangementNetwork(residuesI_L, residuesA_L, 5.0, 0.3)

# For print out we are using the position numbers consistent with the active protein.
for rrContact in crn.rrContacts_L:
    resX = residuesA_L[rrContact.resXix]
    resY = residuesA_L[rrContact.resYix]
    print "%3d %3s   %3d %3s  %6.3f" %(resX.id.position, resX.type,
                                         resY.id.position, resY.type, rrContact.rij)
    












